@@ -1,4 +1,4 @@
-// "$Id: Fl_My_Native_File_Chooser_MAC.mm 10317 2014-09-16 17:34:29Z manolo $"
+// "$Id$"
 //
 // FLTK native OS file chooser widget
 //
@@ -346,11 +346,17 @@ int Fl_My_Native_File_Chooser::get_saveas_basename(void) {
   char *q = strdup( [[[(NSSavePanel*)_panel URL] path] UTF8String] );
   if ( !(_options & SAVEAS_CONFIRM) ) {
     const char *d = [[[[(NSSavePanel*)_panel URL] path] stringByDeletingLastPathComponent] UTF8String];
-    int l = strlen(d) + 1;
+    int l = (int)strlen(d) + 1;
     if (strcmp(d, "/") == 0) l = 1;
     int lu = strlen(UNLIKELYPREFIX);
     // Remove UNLIKELYPREFIX between directory and filename parts
-    memmove(q + l, q + l + lu, strlen(q + l + lu) + 1);
+
+    // Daniel
+    //memmove(q + l, q + l + lu, strlen(q + l + lu) + 1); orig from  fltk //1.3.5
+
+    //fixes file picker on catalina
+    if (memcmp(q+l, UNLIKELYPREFIX, lu) == 0) memmove(q + l, q + l + lu, strlen(q + l + lu) + 1);
+
   }
   set_single_pathname( q );
   free(q);
@@ -532,7 +538,7 @@ int Fl_My_Native_File_Chooser::runmodal()
   NSString *dir = nil;
   NSString *fname = nil;
   NSString *preset = nil;
-  int retval;
+  NSInteger retval;
   if (_preset_file) {
     preset = [[NSString alloc] initWithUTF8String:_preset_file];
     if (strchr(_preset_file, '/') != NULL) {
@@ -551,7 +557,7 @@ int Fl_My_Native_File_Chooser::runmodal()
   }
   [dir release];
   [preset release];
-  return retval;
+  return (retval == NSFileHandlingPanelOKButton ? 1 : 0);
 }
 
 // POST BROWSER
@@ -568,6 +574,7 @@ int Fl_My_Native_File_Chooser::post() {
   if ( _filt_total == 0 ) {	// Make sure they match
     _filt_value = 0;		// TBD: move to someplace more logical?
   }
+  fl_open_display();
   NSAutoreleasePool *localPool;
   localPool = [[NSAutoreleasePool alloc] init];
   switch (_btype) {
@@ -582,8 +589,12 @@ int Fl_My_Native_File_Chooser::post() {
       _panel =  [NSSavePanel savePanel];
       break;
   }
-  NSString *nstitle = [NSString stringWithUTF8String: (_title ? _title : "No Title")];
-  [(NSSavePanel*)_panel setTitle:nstitle];
+  BOOL is_open_panel = [(NSSavePanel*)_panel isKindOfClass:[NSOpenPanel class]];
+  if (_title) {
+    SEL title_or_message = (is_open_panel && fl_mac_os_version >= 101200) ?
+              @selector(setMessage:) : @selector(setTitle:);
+    [(NSSavePanel*)_panel performSelector:title_or_message withObject:[NSString stringWithUTF8String:_title]];
+  }
   switch (_btype) {
     case BROWSE_MULTI_FILE:
       [(NSOpenPanel*)_panel setAllowsMultipleSelection:YES];
@@ -593,9 +604,6 @@ int Fl_My_Native_File_Chooser::post() {
       /* FALLTHROUGH */
     case BROWSE_DIRECTORY:
       [(NSOpenPanel*)_panel setCanChooseDirectories:YES];
-       if (_options & NEW_FOLDER) {
-          [(NSSavePanel*)_panel setCanCreateDirectories:YES];
-      }
       break;
     case BROWSE_SAVE_DIRECTORY:
       [(NSSavePanel*)_panel setCanCreateDirectories:YES];
@@ -604,7 +612,6 @@ int Fl_My_Native_File_Chooser::post() {
   
   // SHOW THE DIALOG
   NSWindow *key = [NSApp keyWindow];
-  BOOL is_open_panel = [(NSSavePanel*)_panel isKindOfClass:[NSOpenPanel class]];
   NSPopUpButton *popup = nil;
   if ( is_open_panel ) {
     if (_filt_total) {
@@ -612,7 +619,7 @@ int Fl_My_Native_File_Chooser::post() {
       popup = createPopupAccessory((NSSavePanel*)_panel, t, Fl_File_Chooser::show_label, 0);
       delete[] t;
       [[popup menu] addItem:[NSMenuItem separatorItem]];
-      [popup addItemWithTitle:[[NSString alloc] initWithUTF8String:Fl_File_Chooser::all_files_label]];
+      [popup addItemWithTitle:[NSString stringWithUTF8String:Fl_File_Chooser::all_files_label]];
       [popup setAction:@selector(validateVisibleColumns)];
       [popup setTarget:(NSObject*)_panel];
       FLopenDelegate *openDelegate = [[[FLopenDelegate alloc] init] autorelease];
@@ -640,27 +647,27 @@ int Fl_My_Native_File_Chooser::post() {
   }
   int retval = runmodal();
   if (_filt_total) {
-    _filt_value = [popup indexOfSelectedItem];
+    _filt_value = (int)[popup indexOfSelectedItem];
   }
-  if ( retval == NSOKButton ) {
-    if (is_open_panel) {
-      clear_pathnames();
-      NSArray *array = [(NSOpenPanel*)_panel URLs];
-      _tpathnames = [array count];
-      _pathnames = new char*[_tpathnames];
-      for(int i = 0; i < _tpathnames; i++) {
-	_pathnames[i] = strnew([[(NSURL*)[array objectAtIndex:i] path] UTF8String]);
+  if ( retval == 1 ) {
+      if (is_open_panel) {
+          clear_pathnames();
+          NSArray *array = [(NSOpenPanel*)_panel URLs];
+          _tpathnames = (int)[array count];
+          _pathnames = new char*[_tpathnames];
+          for(int i = 0; i < _tpathnames; i++) {
+              _pathnames[i] = strnew([[(NSURL*)[array objectAtIndex:i] path] UTF8String]);
+          }
       }
-    }
-    else get_saveas_basename();
+      else get_saveas_basename();
   }
   [key makeKeyWindow];
   [localPool release];
-  return (retval == NSOKButton ? 0 : 1);
+  return (retval == 1 ? 0 : 1);
 }
 
 #endif // __APPLE__
 
 //
-// End of "$Id: Fl_My_Native_File_Chooser_MAC.mm 10317 2014-09-16 17:34:29Z manolo $".
+// End of "$Id$".
 //
