@@ -20,10 +20,11 @@
 #include <time.h>
 #include <sys/stat.h>
 
-#ifndef _WIN32
+#ifndef WIN32
  #include <sys/wait.h>
 #endif
 
+#include "gettext.h"
 #include "config.h"
 
 #include "cfg.h"
@@ -66,11 +67,13 @@ void fill_cfg_widgets(void)
     {
         fl_g->button_cfg_edit_srv->activate();
         fl_g->button_cfg_del_srv->activate();
+        fl_g->choice_cfg_act_srv->activate();
     }
     else
     {
         fl_g->button_cfg_edit_srv->deactivate();
         fl_g->button_cfg_del_srv->deactivate();
+        fl_g->choice_cfg_act_srv->deactivate();
     }
 
     fl_g->choice_cfg_act_icy->clear();
@@ -82,11 +85,14 @@ void fill_cfg_widgets(void)
     {
         fl_g->button_cfg_edit_icy->activate();
         fl_g->button_cfg_del_icy->activate();
+        fl_g->choice_cfg_act_icy->activate();
+
     }
     else
     {
         fl_g->button_cfg_edit_icy->deactivate();
         fl_g->button_cfg_del_icy->deactivate();
+        fl_g->choice_cfg_act_icy->deactivate();
     }
 
     fl_g->choice_cfg_act_srv->value(cfg.selected_srv);
@@ -94,8 +100,12 @@ void fill_cfg_widgets(void)
 
 
     fl_g->check_cfg_connect->value(cfg.main.connect_at_startup);
+    fl_g->check_cfg_force_reconnecting->value(cfg.main.force_reconnecting);
+
     fl_g->input_log_filename->value(cfg.main.log_file);
 
+    fl_g->check_update_at_startup->value(cfg.main.check_for_update);
+
     //fill the audio section
     if(!strcmp(cfg.audio.codec, "mp3"))
         fl_g->choice_cfg_codec->value(CHOICE_MP3);
@@ -118,41 +128,52 @@ void fill_cfg_widgets(void)
         fl_g->choice_cfg_channel->value(CHOICE_STEREO);
 
     for(i = 0; bitrate[i] != 0; i++)
+    {
         if(cfg.audio.bitrate == bitrate[i])
         {
             fl_g->choice_cfg_bitrate->value(i);
             break;
         }
+    }
 
+    fl_g->input_cfg_present_level->value(-cfg.audio.signal_level);
+    fl_g->input_cfg_absent_level->value(-cfg.audio.silence_level);
+
+    fl_g->check_cfg_mono_to_stereo->value(cfg.audio.mono_to_stereo);
+    
     fl_g->input_cfg_buffer->value(cfg.audio.buffer_ms);
 
     fl_g->choice_cfg_resample_mode->value(cfg.audio.resample_mode);
     
     // Fill stream section
-    if(cfg.main.song_update)
-        fl_g->check_song_update_active->value(1);
-    else
-        fl_g->check_song_update_active->value(0);
+    fl_g->check_song_update_active->value(cfg.main.song_update);
+    fl_g->check_read_last_line->value(cfg.main.read_last_line);
 
     fl_g->input_cfg_song_file->value(cfg.main.song_path);
+    fl_g->input_cfg_signal->value(cfg.main.signal_threshold);
+    fl_g->input_cfg_silence->value(cfg.main.silence_threshold);
     
+    fl_g->input_cfg_song_prefix->value(cfg.main.song_prefix);
+    fl_g->input_cfg_song_suffix->value(cfg.main.song_suffix);
+
     
 #if __APPLE__ && __MACH__
-    fl_g->choice_cfg_app->add("iTunes");
+    fl_g->choice_cfg_app->add("iTunes\\/Music");
     fl_g->choice_cfg_app->add("Spotify");
     fl_g->choice_cfg_app->add("VOX");
     fl_g->check_cfg_use_app->value(cfg.main.app_update);
     fl_g->choice_cfg_app->value(cfg.main.app_update_service);
-#elif __linux__ && HAVE_DBUS
+#elif (__linux__ || __FreeBSD__) && HAVE_DBUS
     fl_g->choice_cfg_app->add("Rhythmbox");
     fl_g->choice_cfg_app->add("Banshee");
     fl_g->choice_cfg_app->add("Clementine");
     fl_g->choice_cfg_app->add("Spotify");
     fl_g->choice_cfg_app->add("Cantata");
+    fl_g->choice_cfg_app->add("Strawberry");
     fl_g->check_cfg_use_app->value(cfg.main.app_update);
     fl_g->choice_cfg_app->value(cfg.main.app_update_service);
-#elif _WIN32
-    fl_g->choice_cfg_app->add("Not supported on Windows");
+#elif WIN32
+    fl_g->choice_cfg_app->add(_("Not supported on Windows"));
     fl_g->choice_cfg_app->value(0);
     fl_g->check_cfg_use_app->value(0);
     fl_g->check_cfg_use_app->deactivate();
@@ -190,9 +211,15 @@ void fill_cfg_widgets(void)
             fl_g->choice_rec_bitrate->value(i);
 
     if(cfg.rec.start_rec)
-        fl_g->check_cfg_rec->value(1);
+        fl_g->check_cfg_auto_start_rec->value(1);
     else
-        fl_g->check_cfg_rec->value(0);
+        fl_g->check_cfg_auto_start_rec->value(0);
+    
+    if(cfg.rec.stop_rec)
+        fl_g->check_cfg_auto_stop_rec->value(1);
+    else
+        fl_g->check_cfg_auto_stop_rec->value(0);
+    
 
     if(cfg.rec.rec_after_launch)
         fl_g->check_cfg_rec_after_launch->value(1);
@@ -204,8 +231,17 @@ void fill_cfg_widgets(void)
     else
         fl_g->check_sync_to_full_hour->value(0);
 
+    fl_g->input_rec_signal->value(cfg.rec.signal_threshold);
+    fl_g->input_rec_silence->value(cfg.rec.silence_threshold);
+
+    //fill the ssl/tls section
+    fl_g->input_tls_cert_file->value(cfg.tls.cert_file);
+    fl_g->input_tls_cert_dir->value(cfg.tls.cert_dir);
+
+
     update_samplerates();
-    
+    update_channel_lists();
+
     //fill the DSP section
     fl_g->check_activate_eq->value(cfg.dsp.equalizer);
     
@@ -223,7 +259,24 @@ void fill_cfg_widgets(void)
     
     slider_equalizer5_cb(cfg.dsp.gain5);
     fl_g->equalizerSlider5->value(cfg.dsp.gain5);
+	
+	fl_g->check_activate_drc->value(cfg.dsp.compressor);
+	
+	slider_threshold_cb(cfg.dsp.threshold);
+    fl_g->thresholdSlider->value(cfg.dsp.threshold);
+	
+	slider_ratio_cb(cfg.dsp.ratio);
+    fl_g->ratioSlider->value(cfg.dsp.ratio);
+	
+	slider_attack_cb(cfg.dsp.attack);
+    fl_g->attackSlider->value(cfg.dsp.attack);
+	
+	slider_release_cb(cfg.dsp.release);
+    fl_g->releaseSlider->value(cfg.dsp.release);
     
+	slider_makeup_cb(cfg.dsp.makeup_gain);
+    fl_g->makeupSlider->value(cfg.dsp.makeup_gain);
+	
     //fill the GUI section
     fl_g->button_gui_bg_color->color(cfg.main.bg_color,
             fl_lighter((Fl_Color)cfg.main.bg_color));
@@ -236,9 +289,14 @@ void fill_cfg_widgets(void)
         fl_g->window_main->stay_on_top(1);
         fl_g->window_cfg->stay_on_top(1);
     }
+    fl_g->check_gui_hide_log_window->value(cfg.gui.hide_log_window);
+    fl_g->check_gui_remember_pos->value(cfg.gui.remember_pos);
+
+   
     fl_g->check_gui_lcd_auto->value(cfg.gui.lcd_auto);
     
-    unsaved_changes = false;
+    fl_g->choice_gui_language->value(cfg.gui.lang);
+
 }
 
 //Updates the samplerate drop down menu for the audio
@@ -255,7 +313,7 @@ void update_samplerates(void)
 
     for(i = 0; sr_list[i] != 0; i++)
     {
-        snprintf(sr_asc, 10, "%dHz", sr_list[i]);
+        snprintf(sr_asc, sizeof(sr_asc), "%dHz", sr_list[i]);
         fl_g->choice_cfg_samplerate->add(sr_asc);
 
         if(cfg.audio.samplerate == sr_list[i])
@@ -268,6 +326,31 @@ void update_samplerates(void)
     }
 }
 
+void update_channel_lists(void)
+{
+    
+    int i;
+    char ch_num_txt[8];
+    
+    int dev_num = fl_g->choice_cfg_dev->value();
+    char* dev_name = cfg.audio.pcm_list[dev_num]->name;
+    int num_of_channels = cfg.audio.pcm_list[dev_num]->num_of_channels;
+    
+    fl_g->choice_cfg_left_channel->clear();
+    fl_g->choice_cfg_right_channel->clear();
+    
+    for (i = 1; i <= num_of_channels; i++)
+    {
+        snprintf(ch_num_txt, sizeof(ch_num_txt), "%d", i);
+        fl_g->choice_cfg_left_channel->add(ch_num_txt);
+        fl_g->choice_cfg_right_channel->add(ch_num_txt);
+    }
+    
+    fl_g->choice_cfg_left_channel->value(cfg.audio.left_ch-1);
+    fl_g->choice_cfg_right_channel->value(cfg.audio.right_ch-1);    
+    
+}
+
 void print_info(const char* info, int info_type)
 {
     char timebuf[10];
@@ -306,11 +389,9 @@ void print_info(const char* info, int info_type)
     if ((cfg.main.log_file != NULL) && (strlen(cfg.main.log_file) > 0))
     {
         strftime(logtimestamp, sizeof(logtimestamp), "%Y-%m-%d %H:%M:%S", mytime);
-        log_fd = fopen(cfg.main.log_file, "ab");
+        log_fd = fl_fopen(cfg.main.log_file, "ab");
         if (log_fd != NULL) 
         {
-            // Probably the most confusing part of the whole code ;)
-            
             if(strchr(infotxt, ':'))
                 strrpl(&infotxt, (char*)"\n", (char*)", ", MODE_ALL);
             else
@@ -397,11 +478,19 @@ void test_file_extension(void)
 
 void init_main_gui_and_audio(void)
 {
+    if(cfg.gui.remember_pos)
+        fl_g->window_main->position(cfg.gui.x_pos, cfg.gui.y_pos);
+    
     fl_g->slider_gain->value(util_factor_to_db(cfg.main.gain));
     fl_g->window_main->redraw();
-
+    
     if(cfg.gui.ontop)
         fl_g->window_main->stay_on_top(1);
+    
+    fl_g->button_info->label(_("Hide log"));
+    if(cfg.gui.hide_log_window)
+       button_info_cb();
+          
 
     lame_stream.channel = cfg.audio.channel;
     lame_stream.bitrate = cfg.audio.bitrate;
@@ -461,3 +550,67 @@ void init_main_gui_and_audio(void)
     flac_enc_reinit(&flac_rec);
 }
 
+
+char *ask_user_msg = NULL;
+char *ask_user_hash = NULL;
+int ask_user_has_clicked = 0;
+int ask_user_answer = 0;
+
+void ask_user_reset(void)
+{
+    if (ask_user_msg != NULL)
+    {
+        free(ask_user_msg);
+        ask_user_msg = NULL;
+    }
+    if (ask_user_hash != NULL)
+    {
+        free(ask_user_hash);
+        ask_user_hash = NULL;
+    }
+    ask_user_has_clicked = 0;
+}
+
+int ask_user_get_has_clicked(void)
+{
+    return ask_user_has_clicked;
+}
+
+int ask_user_get_answer(void)
+{
+    return ask_user_answer;
+}
+
+void ask_user_set_msg(char *msg)
+{
+    int len = strlen(msg);
+    ask_user_msg = (char*)calloc(len+1, sizeof(char));
+    strncpy(ask_user_msg, msg, len);
+}
+
+void ask_user_set_hash(char *hash)
+{
+    int len = strlen(hash);
+    ask_user_hash = (char*)calloc(len+1, sizeof(char));
+    strncpy(ask_user_hash, hash, len);
+}
+
+void ask_user_ask(void)
+{
+    if (fl_choice("%s", _("TRUST"), _("No"), NULL, ask_user_msg) == 1)
+    { // No
+        ask_user_answer = IC_ABORT;
+    }
+    else
+    { // TRUST
+	int len = strlen(ask_user_hash);
+        cfg.srv[cfg.selected_srv]->cert_hash =
+            (char*)realloc(cfg.srv[cfg.selected_srv]->cert_hash, len+1);
+        
+        memset(cfg.srv[cfg.selected_srv]->cert_hash, 0, len+1);
+        strncpy(cfg.srv[cfg.selected_srv]->cert_hash, ask_user_hash, len);
+        ask_user_answer = IC_RETRY;
+    }
+    
+    ask_user_has_clicked = 1;
+}
