@@ -18,10 +18,17 @@
 
 void display_info_timer(void*);
 void vu_meter_timer(void*);
+void cmd_timer(void*);
 void is_connected_timer(void*);
 void cfg_win_pos_timer(void*);
 void songfile_timer(void*);
 void split_recording_timer(void*);
+void stream_silence_timer(void*);
+void record_silence_timer(void*);
+void stream_signal_timer(void*);
+void record_signal_timer(void*);
+
+
 void display_rotate_timer(void*);
 void app_timer(void*);
 
