@@ -20,7 +20,7 @@
 
 #ifdef HAVE_LIBFDK_AAC
 
-#if defined(_WIN32) || defined(__APPLE__)
+#if defined(WIN32) || defined(__APPLE__)
  #include "aac_dll.h"
 #else
  #include <fdk-aac/aacenc_lib.h>
