@@ -18,14 +18,16 @@
 #include <limits.h>
 
 #include "config.h"
+#include "gettext.h"
 
 #include "cfg.h"
 #include "butt.h"
 #include "flgui.h"
 #include "util.h"
 #include "fl_funcs.h"
+#include "strfuncs.h"
 
-#ifdef _WIN32
+#ifdef WIN32
  const char CONFIG_FILE[] = "buttrc";
 #else
  const char CONFIG_FILE[] = ".buttrc";
@@ -33,7 +35,6 @@
 
 config_t cfg;
 char *cfg_path;
-bool unsaved_changes;
 
 int cfg_write_file(char *path)
 {
@@ -47,7 +48,7 @@ int cfg_write_file(char *path)
     cfg_fd = fl_fopen(path, "wb");
     if(cfg_fd == NULL)
     {
-        snprintf(info_buf, sizeof(info_buf), "Could not write to file: %s", path);
+        snprintf(info_buf, sizeof(info_buf), _("Could not write to file: %s"), path);
         print_info(cfg_path, 1);
         return 1;
     }
@@ -98,12 +99,50 @@ int cfg_write_file(char *path)
 
     fprintf(cfg_fd, "song_update = %d\n", cfg.main.song_update);
     
+    
+    
+    
+    if (cfg.main.song_prefix != NULL)
+    {
+        char *tmp = strdup(cfg.main.song_prefix);
+        strrpl(&tmp, (char*)" ", (char*)"%20", MODE_ALL);
+        fprintf(cfg_fd, "song_prefix = %s\n", tmp);
+        free(tmp);
+    }
+    else
+        fprintf(cfg_fd, "song_prefix = \n");
+    
+    if (cfg.main.song_suffix != NULL)
+    {
+        char *tmp = strdup(cfg.main.song_suffix);
+        strrpl(&tmp, (char*)" ", (char*)"%20", MODE_ALL);
+        fprintf(cfg_fd, "song_suffix = %s\n", tmp);
+        free(tmp);
+    }
+    else
+        fprintf(cfg_fd, "song_suffix = \n");
+
+
+    fprintf(cfg_fd, "read_last_line = %d\n", cfg.main.read_last_line);
+
+    
     fprintf(cfg_fd, "app_update_service = %d\n", cfg.main.app_update_service);
     fprintf(cfg_fd, "app_update = %d\n", cfg.main.app_update);
 
     fprintf(cfg_fd, "gain = %f\n", cfg.main.gain);
 
+    fprintf(cfg_fd, "silence_threshold = %d\n", cfg.main.silence_threshold);
+    fprintf(cfg_fd, "signal_threshold = %d\n", cfg.main.signal_threshold);
+    fprintf(cfg_fd, "check_for_update = %d\n", cfg.main.check_for_update);
     fprintf(cfg_fd, "connect_at_startup = %d\n", cfg.main.connect_at_startup);
+    fprintf(cfg_fd, "force_reconnecting = %d\n", cfg.main.force_reconnecting);
+
+
+    if (cfg.main.ic_charset != NULL)
+        fprintf(cfg_fd, "ic_charset = %s\n", cfg.main.ic_charset);
+    else
+        fprintf(cfg_fd, "ic_charset = \n");
+        
 
     if (cfg.main.log_file != NULL)
         fprintf(cfg_fd, "log_file = %s\n\n", cfg.main.log_file);
@@ -116,19 +155,31 @@ int cfg_write_file(char *path)
             "samplerate = %d\n"
             "bitrate = %d\n"
             "channel = %d\n"
+            "left_ch = %d\n"
+            "right_ch = %d\n"
             "codec = %s\n"
+            "mono_to_stereo = %d\n"
             "aac_overwrite_aot = %d\n"
             "aac_aot = %d\n"
             "resample_mode = %d\n"
+            "silence_level = %f\n"
+            "signal_level = %f\n"
+            "disable_dithering = %d\n"
             "buffer_ms = %d\n\n",
             cfg.audio.dev_num,
             cfg.audio.samplerate,
             cfg.audio.bitrate,
             cfg.audio.channel,
+            cfg.audio.left_ch,
+            cfg.audio.right_ch,
             cfg.audio.codec,
+            cfg.audio.mono_to_stereo,
             cfg.audio.aac_overwrite_aot,
             cfg.audio.aac_aot,
             cfg.audio.resample_mode,
+            cfg.audio.silence_level,
+            cfg.audio.signal_level,
+            cfg.audio.disable_dithering,
             cfg.audio.buffer_ms
            );
 
@@ -137,21 +188,35 @@ int cfg_write_file(char *path)
             "bitrate = %d\n"
             "codec = %s\n"
             "start_rec = %d\n"
+            "stop_rec = %d\n"
             "rec_after_launch = %d\n"
             "sync_to_hour  = %d\n"
             "split_time = %d\n"
             "filename = %s\n"
+            "silence_threshold = %d\n"
+            "signal_threshold = %d\n"
             "folder = %s\n\n",
             cfg.rec.bitrate,
             cfg.rec.codec,
             cfg.rec.start_rec,
+            cfg.rec.stop_rec,
             cfg.rec.rec_after_launch,
             cfg.rec.sync_to_hour,
             cfg.rec.split_time,
             cfg.rec.filename,
+            cfg.rec.silence_threshold,
+            cfg.rec.signal_threshold,
             cfg.rec.folder
            );
     
+    fprintf(cfg_fd,
+            "[tls]\n"
+            "cert_file = %s\n"
+            "cert_dir = %s\n\n",
+            cfg.tls.cert_file != NULL ? cfg.tls.cert_file : "",
+            cfg.tls.cert_dir  != NULL ? cfg.tls.cert_dir  : ""
+           );
+    
     fprintf(cfg_fd,
             "[dsp]\n"
             "equalizer = %d\n"
@@ -159,41 +224,71 @@ int cfg_write_file(char *path)
             "gain2 = %f\n"
             "gain3 = %f\n"
             "gain4 = %f\n"
-            "gain5 = %f\n\n",
+            "gain5 = %f\n"
+			"compressor = %d\n"
+			"threshold = %f\n"
+			"ratio = %f\n"
+			"attack = %f\n"
+			"release = %f\n"
+			"makeup_gain = %f\n\n",
             cfg.dsp.equalizer,
             cfg.dsp.gain1,
             cfg.dsp.gain2,
             cfg.dsp.gain3,
             cfg.dsp.gain4,
-            cfg.dsp.gain5
+            cfg.dsp.gain5,
+			cfg.dsp.compressor,
+			cfg.dsp.threshold,
+			cfg.dsp.ratio,
+			cfg.dsp.attack,
+			cfg.dsp.release,
+			cfg.dsp.makeup_gain
             );
 
     fprintf(cfg_fd,
             "[gui]\n"
             "attach = %d\n"
             "ontop = %d\n"
-            "lcd_auto = %d\n\n",
+            "hide_log_window = %d\n"
+            "remember_pos = %d\n"
+            "x_pos = %d\n"
+            "y_pos = %d\n"
+            "lcd_auto = %d\n"
+            "lang = %d\n\n",
             cfg.gui.attach,
             cfg.gui.ontop,
-            cfg.gui.lcd_auto
+            cfg.gui.hide_log_window,
+            cfg.gui.remember_pos,
+            cfg.gui.x_pos,
+            cfg.gui.y_pos,
+            cfg.gui.lcd_auto,
+            cfg.gui.lang
            );
 
-    for(i = 0; i < cfg.main.num_of_srv; i++)
+    for (i = 0; i < cfg.main.num_of_srv; i++)
     {
         fprintf(cfg_fd, 
                 "[%s]\n"
                 "address = %s\n"
                 "port = %u\n"
                 "password = %s\n"
-                "type = %d\n",
+                "type = %d\n"
+                "tls = %d\n",
                 cfg.srv[i]->name,
                 cfg.srv[i]->addr,
                 cfg.srv[i]->port,
                 cfg.srv[i]->pwd,
-                cfg.srv[i]->type
+                cfg.srv[i]->type,
+                cfg.srv[i]->tls
                );
+        
+        if (cfg.srv[i]->cert_hash != NULL)
+            fprintf(cfg_fd, "cert_hash = %s\n", cfg.srv[i]->cert_hash);
+        else
+            fprintf(cfg_fd, "cert_hash = \n");
 
-        if(cfg.srv[i]->type == ICECAST)
+
+        if (cfg.srv[i]->type == ICECAST)
         {
             fprintf(cfg_fd, "mount = %s\n", cfg.srv[i]->mount);
             fprintf(cfg_fd, "usr = %s\n\n", cfg.srv[i]->usr);
@@ -203,6 +298,8 @@ int cfg_write_file(char *path)
             fprintf(cfg_fd, "mount = (none)\n");
             fprintf(cfg_fd, "usr = (none)\n\n");
         }
+        
+       
 
     }
 
@@ -249,11 +346,9 @@ int cfg_write_file(char *path)
 
     fclose(cfg_fd);
 
-    snprintf(info_buf, sizeof(info_buf), "Config written to %s", path);
+    snprintf(info_buf, sizeof(info_buf), _("Config written to %s"), path);
     print_info(info_buf, 0);
 
-    unsaved_changes = 0;
-
     return 0;
 }
 
@@ -270,22 +365,26 @@ int cfg_set_values(char *path)
     if(cfg_parse_file(path) == -1)
         return 1;
 
-    unsaved_changes = 0;
-
     cfg.main.log_file    = cfg_get_str("main", "log_file");
+    cfg.main.ic_charset  = cfg_get_str("main", "ic_charset");
 
     cfg.audio.dev_num    = cfg_get_int("audio", "device");
     cfg.audio.samplerate = cfg_get_int("audio", "samplerate");
     cfg.audio.resolution = 16;
     cfg.audio.bitrate    = cfg_get_int("audio", "bitrate");
     cfg.audio.channel    = cfg_get_int("audio", "channel");
+    cfg.audio.left_ch    = cfg_get_int("audio", "left_ch");
+    cfg.audio.right_ch   = cfg_get_int("audio", "right_ch");
     cfg.audio.codec      = cfg_get_str("audio", "codec");
     cfg.audio.buffer_ms  = cfg_get_int("audio", "buffer_ms");
     cfg.audio.aac_aot    = cfg_get_int("audio", "aac_aot");
-    cfg.audio.aac_overwrite_aot = cfg_get_int("audio", "aac_overwrite_aot");
+    
+    cfg.audio.disable_dithering = cfg_get_int("audio", "disable_dithering");
     cfg.audio.pcm_list   = snd_get_devices(&cfg.audio.dev_count);
-
+    
+    cfg.audio.mono_to_stereo = cfg_get_int("audio", "mono_to_stereo");
     cfg.audio.resample_mode  = cfg_get_int("audio", "resample_mode");
+    cfg.audio.aac_overwrite_aot = cfg_get_int("audio", "aac_overwrite_aot");
     
     if(cfg.audio.dev_num < 0)
         cfg.audio.dev_num = 0;
@@ -298,6 +397,15 @@ int cfg_set_values(char *path)
 
     if(cfg.audio.channel == -1)
         cfg.audio.channel = 2;
+    
+    if(cfg.audio.left_ch == -1)
+        cfg.audio.left_ch = 1;
+    
+    if(cfg.audio.right_ch == -1)
+        cfg.audio.right_ch = 2;
+    
+    if(cfg.audio.mono_to_stereo == -1)
+        cfg.audio.mono_to_stereo = 0;
 
 
     if(cfg.audio.codec == NULL)
@@ -311,6 +419,16 @@ int cfg_set_values(char *path)
     if(!strcmp(cfg.audio.codec, "aac") && g_aac_lib_available == 0) {
         strcpy(cfg.audio.codec, "mp3");
     }
+
+
+    cfg.audio.silence_level = cfg_get_float("audio", "silence_level");
+    if(isnan(cfg.audio.silence_level))
+        cfg.audio.silence_level = 50.0; // Will be interpreted as negative value (-50 dB) 
+    
+    cfg.audio.signal_level = cfg_get_float("audio", "signal_level");
+    if(isnan(cfg.audio.signal_level))
+        cfg.audio.signal_level = 50.0;
+    
         
     if(cfg.audio.buffer_ms == -1)
         cfg.audio.buffer_ms = 50;
@@ -318,6 +436,10 @@ int cfg_set_values(char *path)
     if(cfg.audio.aac_overwrite_aot == -1)
         cfg.audio.aac_overwrite_aot = 0;
     
+    if(cfg.audio.disable_dithering == -1)
+        cfg.audio.disable_dithering = 0;
+    
+    
 #ifdef HAVE_LIBFDK_AAC
     aac_stream.overwrite_aot = cfg.audio.aac_overwrite_aot;
     aac_rec.overwrite_aot = cfg.audio.aac_overwrite_aot;
@@ -333,20 +455,27 @@ int cfg_set_values(char *path)
     if(cfg.audio.dev_num > cfg.audio.dev_count - 1)
         cfg.audio.dev_num = 0;
 
-    cfg.rec.bitrate    = cfg_get_int("record", "bitrate");
-    cfg.rec.start_rec  = cfg_get_int("record", "start_rec");
+    cfg.rec.bitrate = cfg_get_int("record", "bitrate");
+    cfg.rec.start_rec = cfg_get_int("record", "start_rec");
+    cfg.rec.stop_rec = cfg_get_int("record", "stop_rec");
     cfg.rec.rec_after_launch  = cfg_get_int("record", "rec_after_launch");
     cfg.rec.sync_to_hour = cfg_get_int("record", "sync_to_hour");
     cfg.rec.split_time = cfg_get_int("record", "split_time");
-    cfg.rec.codec      = cfg_get_str("record", "codec");
-    cfg.rec.filename   = cfg_get_str("record", "filename");
-    cfg.rec.folder     = cfg_get_str("record", "folder");
+    cfg.rec.codec = cfg_get_str("record", "codec");
+    cfg.rec.filename = cfg_get_str("record", "filename");
+    cfg.rec.folder = cfg_get_str("record", "folder");
+    cfg.rec.silence_threshold = cfg_get_int("record", "silence_threshold");
+    cfg.rec.signal_threshold = cfg_get_int("record", "signal_threshold");
+
 
     if(cfg.rec.bitrate == -1)
         cfg.rec.bitrate = 192;
 
     if(cfg.rec.start_rec == -1)
         cfg.rec.start_rec = 0;
+    
+    if(cfg.rec.stop_rec == -1)
+        cfg.rec.stop_rec = 0;
 
     if(cfg.rec.rec_after_launch == -1)
         cfg.rec.rec_after_launch = 0;
@@ -381,14 +510,24 @@ int cfg_set_values(char *path)
         cfg.rec.folder = (char*)malloc(3*sizeof(char));
         strcpy(cfg.rec.folder, "./");
     }
+    
+    if(cfg.rec.silence_threshold == -1)
+        cfg.rec.silence_threshold = 0;
+    
+    if(cfg.rec.signal_threshold == -1)
+        cfg.rec.signal_threshold = 0;
 
+    
+    cfg.tls.cert_file = cfg_get_str("tls", "cert_file");
+    cfg.tls.cert_dir = cfg_get_str("tls", "cert_dir");
+    
 
     cfg.selected_srv = 0;
 
     cfg.main.num_of_srv = cfg_get_int("main", "num_of_srv");
     if(cfg.main.num_of_srv == -1)
     {
-        fl_alert("error while parsing config. Illegal value (-1) for num_of_srv.\nbutt is going to close now");
+        fl_alert(_("error while parsing config. Illegal value (-1) for num_of_srv.\nbutt is going to close now"));
         exit(1);
     }
     if(cfg.main.num_of_srv > 0)
@@ -396,14 +535,14 @@ int cfg_set_values(char *path)
         cfg.main.srv     = cfg_get_str("main", "server");
         if(cfg.main.srv == NULL)
         {
-            fl_alert("error while parsing config. Missing main/server entry.\nbutt is going to close now");
+            fl_alert(_("error while parsing config. Missing main/server entry.\nbutt is going to close now"));
             exit(1);
         }
 
         cfg.main.srv_ent = cfg_get_str("main", "srv_ent");
         if(cfg.main.srv_ent == NULL)
         {
-            fl_alert("error while parsing config. Missing main/srv_ent entry.\nbutt is going to close now");
+            fl_alert(_("error while parsing config. Missing main/srv_ent entry.\nbutt is going to close now"));
             exit(1);
         }
 
@@ -417,55 +556,69 @@ int cfg_set_values(char *path)
 
         for(i = 0; srv_ent != NULL; i++)
         {
-            cfg.srv[i]->name = (char*)malloc((strlen(srv_ent)+1) * sizeof(char));
+            cfg.srv[i]->name = (char*)malloc(strlen(srv_ent)+1);
             snprintf(cfg.srv[i]->name, strlen(srv_ent)+1, "%s", srv_ent);
 
             cfg.srv[i]->addr = cfg_get_str(srv_ent, "address");
             if(cfg.srv[i]->addr == NULL)
             {
-                fl_alert("error while parsing config. Missing address entry for server \"%s\"."
-                        "\nbutt is going to close now", srv_ent);
+                fl_alert(_("error while parsing config. Missing address entry for server \"%s\"."
+                        "\nbutt is going to close now"), srv_ent);
                 exit(1);
             }
 
             cfg.srv[i]->port  = cfg_get_int(srv_ent, "port");
             if(cfg.srv[i]->port == -1)
             {
-                fl_alert("error while parsing config. Missing port entry for server \"%s\"."
-                        "\nbutt is going to close now", srv_ent);
+                fl_alert(_("error while parsing config. Missing port entry for server \"%s\"."
+                        "\nbutt is going to close now"), srv_ent);
                 exit(1);
             }
 
             cfg.srv[i]->pwd   = cfg_get_str(srv_ent, "password");
             if(cfg.srv[i]->pwd == NULL)
             {
-                fl_alert("error while parsing config. Missing password entry for server \"%s\"."
-                        "\nbutt is going to close now", srv_ent);
+                fl_alert(_("error while parsing config. Missing password entry for server \"%s\"."
+                        "\nbutt is going to close now"), srv_ent);
                 exit(1);
             }
 
             cfg.srv[i]->type  = cfg_get_int(srv_ent, "type");
             if(cfg.srv[i]->type == -1)
             {
-                fl_alert("error while parsing config. Missing type entry for server \"%s\"."
-                        "\nbutt is going to close now", srv_ent);
+                fl_alert(_("error while parsing config. Missing type entry for server \"%s\"."
+                        "\nbutt is going to close now"), srv_ent);
                 exit(1);
             }
 
             cfg.srv[i]->mount = cfg_get_str(srv_ent, "mount");
-            if((cfg.srv[i]->mount == NULL) &&  (cfg.srv[i]->type == ICECAST))
+            if((cfg.srv[i]->mount == NULL) && (cfg.srv[i]->type == ICECAST))
             {
-                fl_alert("error while parsing config. Missing mount entry for server \"%s\"."
-                        "\nbutt is going to close now", srv_ent);
+                fl_alert(_("error while parsing config. Missing mount entry for server \"%s\"."
+                        "\nbutt is going to close now"), srv_ent);
                 exit(1);
             }
 
             cfg.srv[i]->usr = cfg_get_str(srv_ent, "usr");
-            if((cfg.srv[i]->usr == NULL) &&  (cfg.srv[i]->type == ICECAST))
+            if((cfg.srv[i]->usr == NULL) && (cfg.srv[i]->type == ICECAST))
             {
-                cfg.srv[i]->usr = (char*)malloc(strlen("source")*sizeof(char)+1);
+                cfg.srv[i]->usr = (char*)malloc(strlen("source")+1);
                 strcpy(cfg.srv[i]->usr, "source");
             }
+#ifdef HAVE_LIBSSL            
+            cfg.srv[i]->tls  = cfg_get_int(srv_ent, "tls");
+#else
+	    cfg.srv[i]->tls = 0;
+#endif
+            if( (cfg.srv[i]->tls != 0) && (cfg.srv[i]->tls != 1) )
+                cfg.srv[i]->tls = 0;
+            
+            cfg.srv[i]->cert_hash = cfg_get_str(srv_ent, "cert_hash");
+            if( (cfg.srv[i]->cert_hash != NULL) && (strlen(cfg.srv[i]->cert_hash) != 64) )
+            {
+                free(cfg.srv[i]->cert_hash);
+                cfg.srv[i]->cert_hash = NULL;
+            }
 
             if(!strcmp(cfg.srv[i]->name, cfg.main.srv))
                 cfg.selected_srv = i;
@@ -477,24 +630,23 @@ int cfg_set_values(char *path)
     cfg.main.num_of_icy = cfg_get_int("main", "num_of_icy");
     if(cfg.main.num_of_icy == -1)
     {
-        fl_alert("error while parsing config. Illegal value (-1) for num_of_icy.\nbutt is going to close now");
+        fl_alert(_("error while parsing config. Illegal value (-1) for num_of_icy.\nbutt is going to close now"));
         exit(1);
     }
     
     cfg.selected_icy = 0;
-
     if(cfg.main.num_of_icy > 0)
     {
         cfg.main.icy     = cfg_get_str("main", "icy");
         if(cfg.main.icy == NULL)
         {
-            fl_alert("error while parsing config. Missing main/icy entry.\nbutt is going to close now");
+            fl_alert(_("error while parsing config. Missing main/icy entry.\nbutt is going to close now"));
             exit(1);
         }
         cfg.main.icy_ent = cfg_get_str("main", "icy_ent");          //icy entries
         if(cfg.main.icy_ent == NULL)
         {
-            fl_alert("error while parsing config. Missing main/icy_ent entry.\nbutt is going to close now");
+            fl_alert(_("error while parsing config. Missing main/icy_ent entry.\nbutt is going to close now"));
             exit(1);
         }
 
@@ -508,7 +660,7 @@ int cfg_set_values(char *path)
 
         for(i = 0; icy_ent != NULL; i++)
         {
-            cfg.icy[i]->name = (char*)malloc(strlen(icy_ent) * sizeof(char)+1);
+            cfg.icy[i]->name = (char*)malloc(strlen(icy_ent)+1);
             snprintf(cfg.icy[i]->name, strlen(icy_ent)+1, "%s", icy_ent);
 
             cfg.icy[i]->desc  = cfg_get_str(icy_ent, "description");
@@ -520,8 +672,8 @@ int cfg_set_values(char *path)
             cfg.icy[i]->pub   = cfg_get_str(icy_ent, "pub");
             if(cfg.icy[i]->pub == NULL)
             {
-                fl_alert("error while parsing config. Missing pub entry for icy \"%s\"."
-                        "\nbutt is going to close now", icy_ent);
+                fl_alert(_("error while parsing config. Missing pub entry for icy \"%s\"."
+                        "\nbutt is going to close now"), icy_ent);
                 exit(1);
             }
 
@@ -536,11 +688,25 @@ int cfg_set_values(char *path)
         free(strtok_buf);
 
     cfg.main.song_path = cfg_get_str("main", "song_path");
+    
+    cfg.main.song_prefix = cfg_get_str("main", "song_prefix");
+    if (cfg.main.song_prefix != NULL)
+        strrpl(&cfg.main.song_prefix, (char*)"%20", (char*)" ", MODE_ALL);
+
+    cfg.main.song_suffix = cfg_get_str("main", "song_suffix");
+    if (cfg.main.song_suffix  != NULL)
+        strrpl(&cfg.main.song_suffix, (char*)"%20", (char*)" ", MODE_ALL);
+
 
     cfg.main.song_update = cfg_get_int("main", "song_update");
     if(cfg.main.song_update == -1)
         cfg.main.song_update = 0; //song update from file is default set to off
     
+    cfg.main.read_last_line = cfg_get_int("main", "read_last_line");
+    if(cfg.main.read_last_line == -1)
+        cfg.main.read_last_line = 0; // read first line per default
+    
+    
     cfg.main.app_update = cfg_get_int("main", "app_update");
     cfg.main.app_update_service = cfg_get_int("main", "app_update_service");
     if(cfg.main.app_update == -1)
@@ -548,17 +714,36 @@ int cfg_set_values(char *path)
     if(cfg.main.app_update_service == -1)
         cfg.main.app_update_service = 0; //Default value, first one
 
+    
+    cfg.main.silence_threshold = cfg_get_int("main", "silence_threshold");
+    if(cfg.main.silence_threshold == -1)
+        cfg.main.silence_threshold = 0;
+    
+    cfg.main.signal_threshold = cfg_get_int("main", "signal_threshold");
+    if(cfg.main.signal_threshold == -1)
+        cfg.main.signal_threshold = 0;
+
 	cfg.main.connect_at_startup = cfg_get_int("main", "connect_at_startup");
 	if(cfg.main.connect_at_startup == -1)
 		cfg.main.connect_at_startup = 0;
+    
+    cfg.main.force_reconnecting = cfg_get_int("main", "force_reconnecting");
+    if(cfg.main.force_reconnecting == -1)
+        cfg.main.force_reconnecting = 0;
+
+
+	cfg.main.check_for_update = cfg_get_int("main", "check_for_update");
+	if(cfg.main.check_for_update == -1)
+		cfg.main.check_for_update = 0;
 
 	cfg.main.gain = cfg_get_float("main", "gain");
-	if((int)cfg.main.gain == -1)
+	if(isnan(cfg.main.gain))
 		cfg.main.gain = 1.0;
     else if(cfg.main.gain > util_db_to_factor(24))
         cfg.main.gain = util_db_to_factor(24);
     else if(cfg.main.gain < 0)
         cfg.main.gain = util_db_to_factor(-24);
+    
 
     // DSP
     cfg.dsp.equalizer = cfg_get_int("dsp", "equalizer");
@@ -566,24 +751,48 @@ int cfg_set_values(char *path)
         cfg.dsp.equalizer = 0;
     
     cfg.dsp.gain1 = cfg_get_float("dsp", "gain1");
-    if (cfg.dsp.gain1 == -1)
+    if (isnan(cfg.dsp.gain1))
         cfg.dsp.gain1 = 0.0;
     
     cfg.dsp.gain2 = cfg_get_float("dsp", "gain2");
-    if (cfg.dsp.gain2 == -1)
+    if (isnan(cfg.dsp.gain2))
         cfg.dsp.gain2 = 0.0;
     
     cfg.dsp.gain3 = cfg_get_float("dsp", "gain3");
-    if (cfg.dsp.gain3 == -1)
+    if (isnan(cfg.dsp.gain3))
         cfg.dsp.gain3 = 0.0;
     
     cfg.dsp.gain4 = cfg_get_float("dsp", "gain4");
-    if (cfg.dsp.gain4 == -1)
+    if (isnan(cfg.dsp.gain4))
         cfg.dsp.gain4 = 0.0;
     
     cfg.dsp.gain5 = cfg_get_float("dsp", "gain5");
-    if (cfg.dsp.gain5 == -1)
+    if (isnan(cfg.dsp.gain5))
         cfg.dsp.gain5 = 0.0;
+	
+	cfg.dsp.compressor = cfg_get_int("dsp", "compressor");
+    if (cfg.dsp.compressor == -1)
+        cfg.dsp.compressor = 0;
+	
+	cfg.dsp.threshold = cfg_get_float("dsp", "threshold");
+    if (isnan(cfg.dsp.threshold))
+        cfg.dsp.threshold = -20.0f; // -20 dBFS
+        
+	cfg.dsp.ratio = cfg_get_float("dsp", "ratio");
+    if (isnan(cfg.dsp.ratio))
+        cfg.dsp.ratio = 5.0;
+	
+	cfg.dsp.attack = cfg_get_float("dsp", "attack");
+    if (isnan(cfg.dsp.attack))
+        cfg.dsp.attack = 0.01; // seconds
+
+	cfg.dsp.release = cfg_get_float("dsp", "release");
+    if (isnan(cfg.dsp.release))
+        cfg.dsp.release = 1.0; // seconds
+	
+	cfg.dsp.makeup_gain = cfg_get_float("dsp", "makeup_gain");
+    if (isnan(cfg.dsp.makeup_gain))
+        cfg.dsp.makeup_gain = 0.0; // 0 dB
 
     
     //read GUI stuff 
@@ -594,11 +803,30 @@ int cfg_set_values(char *path)
     cfg.gui.ontop = cfg_get_int("gui", "ontop");
     if(cfg.gui.ontop == -1)
         cfg.gui.ontop = 0;
+    
+    cfg.gui.hide_log_window = cfg_get_int("gui", "hide_log_window");
+    if(cfg.gui.hide_log_window == -1)
+        cfg.gui.hide_log_window = 0;
+    
+    cfg.gui.remember_pos = cfg_get_int("gui", "remember_pos");
+    if(cfg.gui.remember_pos == -1)
+        cfg.gui.remember_pos = 0;
+    
+    cfg.gui.x_pos = cfg_get_int("gui", "x_pos");
+    if(cfg.gui.x_pos == -1)
+        cfg.gui.x_pos = 0;
+    
+    cfg.gui.y_pos = cfg_get_int("gui", "y_pos");
+    if(cfg.gui.y_pos == -1)
+        cfg.gui.y_pos = 0;
 
     cfg.gui.lcd_auto = cfg_get_int("gui", "lcd_auto");
     if(cfg.gui.lcd_auto == -1)
         cfg.gui.lcd_auto = 0;
-
+    
+    cfg.gui.lang = cfg_get_int("gui", "lang");
+    if(cfg.gui.lang == -1)
+        cfg.gui.lang = LANG_SYSTEM;
 
     //read FLTK related stuff
     cfg.main.bg_color = cfg_get_int("main", "bg_color");
@@ -622,21 +850,20 @@ int cfg_create_default(void)
     if(cfg_fd == NULL)
         return 1;
 
-
-#ifdef _WIN32
-    p = getenv("USERPROFILE");
+#ifdef WIN32
+    p = fl_getenv("USERPROFILE");
     if (p != NULL)
         snprintf(def_rec_folder, PATH_MAX, "%s\\Music\\", p);
     else
         snprintf(def_rec_folder, PATH_MAX, "./");
 #elif __APPLE__
-    p = getenv("HOME");
+    p = fl_getenv("HOME");
     if (p != NULL)
         snprintf(def_rec_folder, PATH_MAX, "%s/Music/", p);
     else
         snprintf(def_rec_folder, PATH_MAX, "./");
 #else //UNIX
-    p = getenv("HOME");
+    p = fl_getenv("HOME");
     if (p != NULL)
         snprintf(def_rec_folder, PATH_MAX, "%s/", p);
     else
@@ -655,8 +882,16 @@ int cfg_create_default(void)
             "icy_ent =\n"
             "song_path =\n"
             "song_update = 0\n"
+            "song_prefix = \n"
+            "song_suffix = \n"
+            "read_last_line = 0\n"
             "log_file =\n"
             "gain = 1.0\n"
+            "ic_charset =\n"
+            "silence_threshhold = 0\n"
+            "signal_threshhold = 0\n"
+            "check_for_update = 0\n"
+            "force_reconnecting = 0\n"
             "connect_at_startup = 0\n\n"
            );
 
@@ -667,9 +902,13 @@ int cfg_create_default(void)
             "bitrate = 128\n"
             "channel = 2\n"
             "codec = mp3\n"
+            "mono_to_stereo = 0\n"
             "resample_mode = 0\n" //SRC_SINC_BEST_QUALITY
             "aac_aot = 5\n" // aac+ v1
             "aac_overwrite_aot = 0\n"
+            "silence_level = 50.0\n"
+            "signal_level = 50.0\n"
+            "disable_dithering = 0\n"
             "buffer_ms = 50\n\n"
            );
 
@@ -680,13 +919,22 @@ int cfg_create_default(void)
             "channel = 2\n"
             "codec = mp3\n"
             "start_rec = 0\n"
+            "stop_rec = 0\n"
             "rec_after_launch = 0\n"
             "sync_to_hour = 0\n"
             "split_time = 0\n"
             "filename = rec_%%Y%%m%%d-%%H%%M%%S.mp3\n"
+            "silence_threshhold = 0\n"
+            "signal_threshhold = 0\n"
             "folder = %s\n\n", def_rec_folder
            );
     
+    fprintf(cfg_fd,
+            "[tls]\n"
+            "cert_file =\n"
+            "cert_dir =\n\n"
+           );
+    
     fprintf(cfg_fd,
             "[dsp]\n"
             "equalizer = 0\n"
@@ -694,13 +942,25 @@ int cfg_create_default(void)
             "gain2 = 0.0\n"
             "gain3 = 0.0\n"
             "gain4 = 0.0\n"
-            "gain5 = 0.0\n");
+            "gain5 = 0.0\n"
+		    "compressor = 0\n"
+			"threshold = -20.0\n"
+			"ratio = 5\n"
+			"attack = 0.01\n"
+			"release = 1.0\n"
+			"makeup_gain = 0.0\n"		
+	);
 
     fprintf(cfg_fd, 
             "[gui]\n"
             "attach = 0\n"
             "ontop = 0\n"
-            "lcd_auto = 0\n\n"
+            "hide_log_window = 0\n"
+            "remember_pos = 0\n"
+            "x_pos = 0\n"
+            "y_pos = 0\n"
+            "lcd_auto = 0\n"
+            "lang = 0\n\n"
             );
 
     fclose(cfg_fd);
