@@ -16,10 +16,17 @@
 #ifndef ICECAST_H
 #define ICECAST_H
 
+enum {
+    IC_OK = 0,
+    IC_RETRY = 1,
+    IC_ABORT = 2,
+    IC_ASK = 3,
+};
+
 int ic_init(void);
 int ic_connect(void);
 int ic_send(char *buf, int buf_len);
-int ic_update_song(void);
+int ic_update_song(char *song_name);
 void ic_disconnect(void);
 
 #endif
