@@ -16,6 +16,8 @@
 #include <stdlib.h>
 #include <lame/lame.h>
 
+#include "gettext.h"
+
 #include "lame_encode.h"
 
 #include "fl_funcs.h"
@@ -35,7 +37,7 @@ int lame_enc_init(lame_enc *lame)
     if((rc = lame_init_params(lame->gfp)) < 0)
     {
         snprintf(info_buf, sizeof(info_buf),
-                "unable to init lame params %d", rc);
+                _("unable to init lame params %d"), rc);
 
         print_info(info_buf, 1);
         return 1;
