@@ -182,8 +182,7 @@ int opus_enc_init(opus_enc *opus)
 	opus_encoder_ctl (opus->encoder, OPUS_GET_LOOKAHEAD (&opus->header->preskip));
 	opus->header_size = opus_header_to_packet (opus->header, opus->header_data, 100);
 
-	opus->tags_size = 
-	8 + 4 + strlen (opus_get_version_string ()) + 4 + 4 + strlen ("ENCODER=") + strlen (VERSION);
+	opus->tags_size = 8 + 4 + strlen (opus_get_version_string ()) + 4 + 4 + strlen ("ENCODER=") + strlen (PACKAGE_STRING);
 	
 	memcpy (opus->tags, "OpusTags", 8);
 	
@@ -193,13 +192,13 @@ int opus_enc_init(opus_enc *opus)
 
 	opus->tags[12 + strlen (opus_get_version_string ())] = 1;
 
-	opus->tags[12 + strlen (opus_get_version_string ()) + 4] = strlen ("ENCODER=") + strlen (VERSION);
+	opus->tags[12 + strlen (opus_get_version_string ()) + 4] = strlen ("ENCODER=") + strlen (PACKAGE_STRING);
 	
 	memcpy (opus->tags + 12 + strlen (opus_get_version_string ()) + 4 + 4, "ENCODER=", strlen ("ENCODER="));
 	
 	memcpy (opus->tags + 12 + strlen (opus_get_version_string ()) + 4 + 4 + strlen ("ENCODER="),
-			VERSION,
-			strlen (VERSION));	
+			PACKAGE_STRING,
+			strlen (PACKAGE_STRING));	
 
 	//printf("Opus Encoder Created\n");
 
@@ -256,7 +255,7 @@ int opus_enc_encode(opus_enc *opus, short *pcm_buf, char *enc_buf, int size)
         return 0;
 
 	if (opus->encoder == NULL) {
-		printf("Opus Encoder NULL wtf?\n");
+		//printf("Opus Encoder NULL wtf?\n");
 		return 0;
 	}
 	
