@@ -23,10 +23,11 @@
 #include <pthread.h>
 #include <samplerate.h>
 
-#ifdef _WIN32
+#ifdef WIN32
  #include <windows.h>
 #endif
 
+#include "gettext.h"
 #include "config.h"
 
 #include "butt.h"
@@ -45,7 +46,7 @@
 #include "fl_funcs.h"
 #include "dsp.hpp"
 
-int pa_frames = 2048;
+int pa_frames;
 
 char* encode_buf;
 short *pa_pcm_buf;
@@ -56,6 +57,10 @@ int framepacket_size;
 bool try_to_connect;
 bool pa_new_frames;
 bool reconnect;
+bool silence_detected;
+bool signal_detected;
+
+int num_of_input_channels;
 
 bool next_file;
 FILE *next_fd;
@@ -85,28 +90,30 @@ int snd_init(void)
     if((p_err = Pa_Initialize()) != paNoError)
     {
         snprintf(info_buf, sizeof(info_buf),
-				"PortAudio init failed:\n%s\n",
+				_("PortAudio init failed:\n%s\n"),
 				Pa_GetErrorText(p_err));
 
         ALERT(info_buf);
         return 1;
     }
 
-    srconv_stream.data_in = (float*)malloc(pa_frames*2 * sizeof(float));
-    srconv_stream.data_out = (float*)malloc(pa_frames*2 * 10 * sizeof(float));
-    srconv_record.data_in = (float*)malloc(pa_frames*2 * sizeof(float));
-    srconv_record.data_out = (float*)malloc(pa_frames*2 * 10 * sizeof(float));
+    srconv_stream.data_in = (float*)malloc(2*pa_frames * 64 * sizeof(float));
+    srconv_record.data_in = (float*)malloc(2*pa_frames * 64 * sizeof(float));
 
-    reconnect = 0;
+    srconv_stream.data_out = (float*)malloc(2*pa_frames * 64 * sizeof(float));
+    srconv_record.data_out = (float*)malloc(2*pa_frames * 64 * sizeof(float));
+
+    reconnect = false;
     buf_index = 0;
+    silence_detected = false;
     return 0;
 }
 
 void snd_reinit(void)
 {
     snd_close();
-    snd_init();
-    snd_open_stream();
+    if (snd_init() == 0)
+        snd_open_stream();
 }
 
 int snd_open_stream(void)
@@ -122,25 +129,16 @@ int snd_open_stream(void)
 
     if(cfg.audio.dev_count == 0)
     {
-        print_info("ERROR: no sound device with input channels found", 1);
+        print_info(_("ERROR: no sound device with input channels found"), 1);
         return 1;
     }
 
 
-    pa_frames = (cfg.audio.buffer_ms/1000.0)*cfg.audio.samplerate;
+    pa_frames = round((cfg.audio.buffer_ms/1000.0)*cfg.audio.samplerate);
 
     snd_reset_samplerate_conv(SND_STREAM);
     snd_reset_samplerate_conv(SND_REC);
 
-
-    framepacket_size = pa_frames * cfg.audio.channel;
-
-    pa_pcm_buf = (short*)malloc(16 * framepacket_size * sizeof(short));
-    encode_buf = (char*)malloc(16 * framepacket_size * sizeof(char));
-
-    rb_init(&rec_rb, 16 * framepacket_size * sizeof(short));
-    rb_init(&stream_rb, 16 * framepacket_size * sizeof(short));
-
     samplerate = cfg.audio.samplerate;
 
     pa_dev_id = cfg.audio.pcm_list[cfg.audio.dev_num]->dev_id;
@@ -148,16 +146,37 @@ int snd_open_stream(void)
     pa_dev_info = Pa_GetDeviceInfo(pa_dev_id);
     if(pa_dev_info == NULL)
     {
-        snprintf(info_buf, 127, "Error getting device Info (%d)", pa_dev_id);
+        snprintf(info_buf, 127, _("Error getting device Info (%d)"), pa_dev_id);
         print_info(info_buf, 1);
         return 1;
     }
+    
+    num_of_input_channels = pa_dev_info->maxInputChannels;
+    if (num_of_input_channels == 1)
+    {
+        cfg.audio.left_ch = 1;
+        cfg.audio.right_ch = 1;
+    }
+    else if ((cfg.audio.left_ch > num_of_input_channels) || (cfg.audio.right_ch > num_of_input_channels))
+    {
+        cfg.audio.left_ch = 1;
+        cfg.audio.right_ch = 2;
+    }
+   
+    framepacket_size = pa_frames * cfg.audio.channel;
+
+    pa_pcm_buf = (short*)malloc(16 * framepacket_size * sizeof(short));
+    encode_buf = (char*)malloc(16 * framepacket_size * sizeof(char));
+
+    rb_init(&rec_rb, 16 * framepacket_size * sizeof(short));
+    rb_init(&stream_rb, 16 * framepacket_size * sizeof(short));
 
     pa_params.device = pa_dev_id;
-    pa_params.channelCount = cfg.audio.channel;
+    pa_params.channelCount = pa_dev_info->maxInputChannels;
     pa_params.sampleFormat = paInt16;
     pa_params.suggestedLatency = pa_dev_info->defaultHighInputLatency;
     pa_params.hostApiSpecificStreamInfo = NULL;
+    
 
     pa_err = Pa_IsFormatSupported(&pa_params, NULL, samplerate);
     if(pa_err != paFormatIsSupported)
@@ -165,8 +184,8 @@ int snd_open_stream(void)
         if(pa_err == paInvalidSampleRate)
         {
             snprintf(info_buf, sizeof(info_buf),
-                    "Samplerate not supported: %dHz\n"
-                    "Using default samplerate: %dHz",
+                    _("Samplerate not supported: %dHz\n"
+                    "Using default samplerate: %dHz"),
                     samplerate, (int)pa_dev_info->defaultSampleRate);
             print_info(info_buf, 1);
 
@@ -185,23 +204,28 @@ int snd_open_stream(void)
         }
         else
         {
-            snprintf(info_buf, sizeof(info_buf), "PA: Format not supported: %s\n",
+            snprintf(info_buf, sizeof(info_buf), _("PA: Format not supported: %s\n"),
                     Pa_GetErrorText(pa_err));
             print_info(info_buf, 1);
             return 1;
         }
     }
 
-    pa_err = Pa_OpenStream(&stream, &pa_params, NULL,
-                            samplerate, pa_frames,
-                            paNoFlag, snd_callback, NULL);
+    int flag = cfg.audio.disable_dithering == 0 ? paNoFlag : paDitherOff;
+    pa_err = Pa_OpenStream(&stream, &pa_params, NULL, samplerate, pa_frames, flag, snd_callback, NULL);
 
     if(pa_err != paNoError)
     {
-        printf("error opening sound device: \n%s\n", Pa_GetErrorText(pa_err));
+        printf(_("error opening sound device: \n%s\n"), Pa_GetErrorText(pa_err));
         return 1;
     }
 
+    if (dsp != NULL)
+    {
+        delete dsp;
+        dsp = NULL;
+    }
+    
     dsp = new DSPEffects(pa_frames, cfg.audio.channel, samplerate);
 
     
@@ -209,8 +233,133 @@ int snd_open_stream(void)
     return 0;
 }
 
+
+//this function is called by PortAudio when new audio data arrived
+int snd_callback(const void *input,
+                 void *output,
+                 unsigned long frameCount,
+                 const PaStreamCallbackTimeInfo* timeInfo,
+                 PaStreamCallbackFlags statusFlags,
+                 void *userData)
+{
+    int samplerate_out;
+    bool convert_stream = false;
+    bool convert_record = false;
+    
+    char stream_buf[16 * pa_frames*2 * sizeof(short)];
+    char record_buf[16 * pa_frames*2 * sizeof(short)];
+
+    short *pcm_input = (short*)input;
+    
+    for (int i = 0; i < frameCount; i++)
+    {
+        if (cfg.audio.channel == 1)
+        {
+            if (num_of_input_channels == 1)
+            {
+                pa_pcm_buf[i] = pcm_input[i];
+            }
+            else
+            {
+                float left_sample, right_sample;
+                short mono_sample;
+                left_sample = (float)pcm_input[num_of_input_channels*i+(cfg.audio.left_ch-1)];
+                right_sample = (float)pcm_input[num_of_input_channels*i+(cfg.audio.right_ch-1)];
+                mono_sample = (short)(round((left_sample+right_sample)/2.0));
+                pa_pcm_buf[i] = mono_sample;
+            }
+        }
+        else {
+            if (num_of_input_channels == 1)
+            {
+                pa_pcm_buf[2*i] = pcm_input[i];
+                pa_pcm_buf[2*i+1] = pcm_input[i];
+            }
+            else
+            {
+                pa_pcm_buf[2*i] = pcm_input[num_of_input_channels*i+(cfg.audio.left_ch-1)];
+                pa_pcm_buf[2*i+1] = pcm_input[num_of_input_channels*i+(cfg.audio.right_ch-1)];
+            }
+        }
+    }
+    samplerate_out = cfg.audio.samplerate;
+
+    if(dsp->hasToProcessSamples())
+    {
+        dsp->processSamples(pa_pcm_buf);
+    }
+    
+    if (streaming)
+    {
+        if ((!strcmp(cfg.audio.codec, "opus")) && (cfg.audio.samplerate != 48000))
+        {
+            convert_stream = true;
+            samplerate_out = 48000;
+        }
+
+        if (convert_stream == true)
+        {
+            srconv_stream.end_of_input = 0;
+            srconv_stream.src_ratio = (float)samplerate_out/cfg.audio.samplerate;
+            srconv_stream.input_frames = frameCount;
+            srconv_stream.output_frames = frameCount*cfg.audio.channel * (srconv_stream.src_ratio+1) * sizeof(float);
+
+            src_short_to_float_array((short*)pa_pcm_buf, (float*)srconv_stream.data_in, frameCount*cfg.audio.channel);
+
+            //The actual resample process
+            src_process(srconv_state_stream, &srconv_stream);
+
+            src_float_to_short_array(srconv_stream.data_out, (short*)stream_buf, srconv_stream.output_frames_gen*cfg.audio.channel);
+
+            rb_write(&stream_rb, (char*)stream_buf, srconv_stream.output_frames_gen*sizeof(short)*cfg.audio.channel);
+        }
+        else
+            rb_write(&stream_rb, (char*)pa_pcm_buf, frameCount*sizeof(short)*cfg.audio.channel);
+
+        pthread_cond_signal(&stream_cond);
+    }
+
+    if(recording)
+    {
+
+        if ((!strcmp(cfg.rec.codec, "opus")) && (cfg.audio.samplerate != 48000))
+        {
+            convert_record = true;
+            samplerate_out = 48000;
+        }
+
+        if (convert_record == true)
+        {
+            srconv_record.end_of_input = 0;
+            srconv_record.src_ratio = (float)samplerate_out/cfg.audio.samplerate;
+            srconv_record.input_frames = frameCount;
+            srconv_record.output_frames = frameCount*cfg.audio.channel * (srconv_record.src_ratio+1) * sizeof(float);
+
+            src_short_to_float_array((short*)pa_pcm_buf, (float*)srconv_record.data_in, frameCount*cfg.audio.channel);
+
+            //The actual resample process
+            src_process(srconv_state_record, &srconv_record);
+
+            src_float_to_short_array(srconv_record.data_out, (short*)record_buf, srconv_record.output_frames_gen*cfg.audio.channel);
+
+            rb_write(&rec_rb, (char*)record_buf, srconv_record.output_frames_gen*sizeof(short)*cfg.audio.channel);
+
+        }
+        else
+            rb_write(&rec_rb, (char*)pa_pcm_buf, frameCount*sizeof(short)*cfg.audio.channel);
+
+        pthread_cond_signal(&rec_cond);
+    }
+    
+    //tell snd_update_vu() that there is new audio data
+    pa_new_frames = 1;
+
+    return 0;
+}
+
+
 void snd_start_stream(void)
-{   
+{
     pthread_mutex_init(&stream_mut, NULL);
     pthread_cond_init (&stream_cond, NULL);
     
@@ -220,19 +369,6 @@ void snd_start_stream(void)
     pthread_create(&stream_thread, NULL, snd_stream_thread, NULL);
 }
 
-void snd_stop_stream(void)
-{
-    connected = 0;
-    streaming = 0;
-
-    pthread_cond_signal(&stream_cond);
-
-    pthread_mutex_destroy(&stream_mut);
-    pthread_cond_destroy(&stream_cond);
-
-
-    print_info("user disconnected\n", 0);
-}
 
 void *snd_stream_thread(void *data)
 {
@@ -246,16 +382,16 @@ void *snd_stream_thread(void *data)
 
     int (*xc_send)(char *buf, int buf_len) = NULL;
 
+    static int new_stream = 0;
 
 
     if(cfg.srv[cfg.selected_srv]->type == SHOUTCAST)
         xc_send = &sc_send;
     else //Icecast
         xc_send = &ic_send;
-    
+        
     while(connected)
     {
-
         pthread_cond_wait(&stream_cond, &stream_mut);
         if(!connected)
             break;
@@ -280,30 +416,29 @@ void *snd_stream_thread(void *data)
                     connected = 0;
                 }
                 else
-                    kbytes_sent += bytes_to_read/1024.0;
-
+                    kbytes_sent += encode_bytes_read/1024.0;
             }
         }
 #ifdef HAVE_LIBFDK_AAC
-	else if(!strcmp(cfg.audio.codec, "aac"))
-	{
-	    bytes_to_read = aac_stream.info.frameLength * cfg.audio.channel * sizeof(short);
-	    while ((rb_filled(&stream_rb)) >= bytes_to_read)
-	    {
-		rb_read_len(&stream_rb, audio_buf, bytes_to_read);
-
-		encode_bytes_read = aac_enc_encode(&aac_stream, (short*)audio_buf, enc_buf,
-			bytes_to_read/(2*cfg.audio.channel), stream_rb.size*10);
-
-		if((sent = xc_send(enc_buf, encode_bytes_read)) == -1)
-		{
-		    connected = 0;
-		}
-		else
-		    kbytes_sent += bytes_to_read/1024.0;
-
-	    }
-	}
+        else if(!strcmp(cfg.audio.codec, "aac"))
+        {
+            bytes_to_read = aac_stream.info.frameLength * cfg.audio.channel * sizeof(short);
+            while ((rb_filled(&stream_rb)) >= bytes_to_read)
+            {
+                rb_read_len(&stream_rb, audio_buf, bytes_to_read);
+                
+                encode_bytes_read = aac_enc_encode(&aac_stream, (short*)audio_buf, enc_buf,
+                                                   bytes_to_read/(2*cfg.audio.channel), stream_rb.size*10);
+                
+                if((sent = xc_send(enc_buf, encode_bytes_read)) == -1)
+                {
+                    connected = 0;
+                }
+                else
+                    kbytes_sent += encode_bytes_read/1024.0;
+                
+            }
+        }
 #endif
         else // ogg, mp3 and flac need more data than opus in order to compress the audio data
         {
@@ -324,30 +459,48 @@ void *snd_stream_thread(void *data)
             
             if(!strcmp(cfg.audio.codec, "flac"))
             {
-                encode_bytes_read = flac_enc_encode_stream(&flac_stream, (short*)audio_buf, (uint8_t*)enc_buf, rb_bytes_read/sizeof(short)/cfg.audio.channel, cfg.audio.channel);
-                
+                encode_bytes_read = flac_enc_encode_stream(&flac_stream, (short*)audio_buf, (uint8_t*)enc_buf, 
+                                                            rb_bytes_read/sizeof(short)/cfg.audio.channel, 
+                                                            cfg.audio.channel, new_stream);
+
+                if (flac_stream.state == FLAC_STATE_UPDATE_META_DATA) 
+                    flac_enc_init_ogg_stream(&flac_stream);
+
                 if (encode_bytes_read == 0)
                     continue;
             }
-
+            
             if((sent = xc_send(enc_buf, encode_bytes_read)) == -1)
                 connected = 0; 
             else
                 kbytes_sent += encode_bytes_read/1024.0;
-
         }
     }
 
-    
     free(enc_buf);
     free(audio_buf);
 
     return NULL;
 }
 
+void snd_stop_stream(void)
+{
+    connected = 0;
+    streaming = 0;
+
+    pthread_cond_signal(&stream_cond);
+
+    pthread_mutex_destroy(&stream_mut);
+    pthread_cond_destroy(&stream_cond);
+
+
+    print_info(_("disconnected\n"), 0);
+}
+
+
+
 void snd_start_rec(void)
 {
-    int error;
     next_file = 0;
 
     kbytes_written = 0;
@@ -358,7 +511,7 @@ void snd_start_rec(void)
 
     pthread_create(&rec_thread, NULL, snd_rec_thread, NULL);
 
-    print_info("recording to:", 0);
+    print_info(_("recording to:"), 0);
     print_info(cfg.rec.path, 0);
 }
 
@@ -373,7 +526,7 @@ void snd_stop_rec(void)
     pthread_mutex_destroy(&rec_mut);
     pthread_cond_destroy(&rec_cond);
 
-    print_info("recording stopped", 0);
+    print_info(_("recording stopped"), 0);
 }
 
 //The recording stuff runs in its own thread
@@ -381,13 +534,11 @@ void snd_stop_rec(void)
 //bandwidth is smaller than the selected streaming bitrate
 void* snd_rec_thread(void *data)
 {
-    int error;
     int rb_bytes_read;
     int bytes_to_read;
     int ogg_header_written;
     int opus_header_written;
     int enc_bytes_read;
-    char info_buf[256];
     
     char *enc_buf = (char*)malloc(rec_rb.size * sizeof(char)*10);
     char *audio_buf = (char*)malloc(rec_rb.size * sizeof(char)*10);
@@ -503,8 +654,8 @@ void* snd_rec_thread(void *data)
 
             if(!strcmp(cfg.rec.codec, "wav"))
             {
-                //this permanently updates the filesize value in the WAV header
-                //so we still have a valid WAV file in case of a crash
+                //This does permanently update the WAV header
+                //so in case of a crash we still have a valid WAV file
                 wav_write_header(cfg.rec.fd, cfg.audio.channel, cfg.audio.samplerate, 16);
                 kbytes_written += fwrite(audio_buf, sizeof(char), rb_bytes_read, cfg.rec.fd)/1024.0;
             }
@@ -522,105 +673,17 @@ void* snd_rec_thread(void *data)
     return NULL;
 }
 
-//this function is called by PortAudio when new audio data arrived
-int snd_callback(const void *input,
-                 void *output,
-                 unsigned long frameCount,
-                 const PaStreamCallbackTimeInfo* timeInfo,
-                 PaStreamCallbackFlags statusFlags,
-                 void *userData)
-{
-    int i;
-    int error;
-    int samplerate_out;
-    bool convert_stream = false;
-    bool convert_record = false;
-    
-    char stream_buf[16 * pa_frames*2 * sizeof(short)];
-    char record_buf[16 * pa_frames*2 * sizeof(short)];
-
-
-    memcpy(pa_pcm_buf, input, frameCount*cfg.audio.channel*sizeof(short));
-    samplerate_out = cfg.audio.samplerate;
-
-    if(dsp->hasToProcessSamples()) {
-        dsp->processSamples(pa_pcm_buf);
-    }
-	
-	if (streaming)
-	{
-        if ((!strcmp(cfg.audio.codec, "opus")) && (cfg.audio.samplerate != 48000))
-        {
-            convert_stream = true;
-            samplerate_out = 48000;
-        }
-
-        if (convert_stream == true)
-        {
-            srconv_stream.end_of_input = 0;
-            srconv_stream.src_ratio = (float)samplerate_out/cfg.audio.samplerate;
-            srconv_stream.input_frames = frameCount;
-            srconv_stream.output_frames = frameCount*cfg.audio.channel * (srconv_stream.src_ratio+1) * sizeof(float);
-
-            src_short_to_float_array((short*)pa_pcm_buf, (float*)srconv_stream.data_in, frameCount*cfg.audio.channel);
-
-            //The actual resample process
-            src_process(srconv_state_stream, &srconv_stream);
-
-            src_float_to_short_array(srconv_stream.data_out, (short*)stream_buf, srconv_stream.output_frames_gen*cfg.audio.channel);
-
-            rb_write(&stream_rb, (char*)stream_buf, srconv_stream.output_frames_gen*sizeof(short)*cfg.audio.channel);
-        }
-        else
-            rb_write(&stream_rb, (char*)pa_pcm_buf, frameCount*sizeof(short)*cfg.audio.channel);
-
-		pthread_cond_signal(&stream_cond);
-	}
-
-	if(recording)
-	{
-
-        if ((!strcmp(cfg.rec.codec, "opus")) && (cfg.audio.samplerate != 48000))
-        {
-            convert_record = true;
-            samplerate_out = 48000;
-        }
-
-        if (convert_record == true)
-        {
-            srconv_record.end_of_input = 0;
-            srconv_record.src_ratio = (float)samplerate_out/cfg.audio.samplerate;
-            srconv_record.input_frames = frameCount;
-            srconv_record.output_frames = frameCount*cfg.audio.channel * (srconv_record.src_ratio+1) * sizeof(float);
-
-            src_short_to_float_array((short*)pa_pcm_buf, (float*)srconv_record.data_in, frameCount*cfg.audio.channel);
-
-            //The actual resample process
-            src_process(srconv_state_record, &srconv_record);
-
-            src_float_to_short_array(srconv_record.data_out, (short*)record_buf, srconv_record.output_frames_gen*cfg.audio.channel);
-
-            rb_write(&rec_rb, (char*)record_buf, srconv_record.output_frames_gen*sizeof(short)*cfg.audio.channel);
-
-        }
-        else
-            rb_write(&rec_rb, (char*)pa_pcm_buf, frameCount*sizeof(short)*cfg.audio.channel);
-
-		pthread_cond_signal(&rec_cond);
-	}
-    
-    //tell vu_update() that there is new audio data
-    pa_new_frames = 1;
-
-    return 0;
-}
 
 void snd_update_vu(void)
 {
     int i;
-    int lpeak = 0;
-    int rpeak = 0;
+    static int lpeak = 0;
+    static int rpeak = 0;
+    static int call_cnt = 0;
     short *p;
+    float decay = 0.5;
+    static double lavg = 0;
+    static double ravg = 0;
 
     p = pa_pcm_buf;
     for(i = 0; i < framepacket_size; i += cfg.audio.channel)
@@ -631,7 +694,35 @@ void snd_update_vu(void)
             rpeak = abs(p[i+(cfg.audio.channel-1)]);
     }
 
-    vu_meter(lpeak, rpeak);
+    int mean_peak = lpeak/2 + rpeak/2;
+    float mean_peak_dB = 20*log10(mean_peak/32768.0);
+
+    if (mean_peak_dB < -cfg.audio.silence_level)
+        silence_detected = true;
+    else
+        silence_detected = false;
+    
+    if (mean_peak_dB > -cfg.audio.signal_level)
+        signal_detected = true;
+    else
+        signal_detected = false;
+    
+    lavg = (decay * lpeak) + (1.0-decay) * lavg;
+    ravg = (decay * rpeak) + (1.0-decay) * ravg;
+
+    // Update the vu meter UI only every second call of this function.
+    // This reduces the CPU usage by not missing any samples
+    if (call_cnt == 1)
+    {
+        vu_meter((short)round(lavg), (short)round(ravg));
+        call_cnt = 0;
+    }
+    else
+    {
+        call_cnt++;
+        lpeak = 0;
+        rpeak = 0;
+    }
 
     pa_new_frames = 0;
 }
@@ -645,11 +736,10 @@ snd_dev_t **snd_get_devices(int *dev_count)
     char info_buf[256];
     PaStreamParameters pa_params;
 
-    int sr[] = { 8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000 };
+    int sr[] = { 8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000};
 
     snd_dev_t **dev_list;
 
-	dev_num = 0;
 
     dev_list = (snd_dev_t**)malloc(100*sizeof(snd_dev_t*));
 
@@ -657,10 +747,11 @@ snd_dev_t **snd_get_devices(int *dev_count)
     for(i = 0; i < 100; i++)
         dev_list[i] = (snd_dev_t*)malloc(sizeof(snd_dev_t));
 
-    dev_list[dev_num]->name = (char*) malloc(strlen("Default PCM device (default)")+1);
-    strcpy(dev_list[dev_num]->name, "Default PCM device (default)");
-    dev_list[dev_num]->dev_id = Pa_GetDefaultInputDevice();
-    dev_num++;
+    dev_list[0]->name = (char*) malloc(strlen(_("Default PCM device (default)"))+1);
+    strcpy(dev_list[0]->name, _("Default PCM device (default)"));
+    dev_list[0]->dev_id = Pa_GetDefaultInputDevice();
+    
+    dev_num = 1;
 
 
     devcount = Pa_GetDeviceCount();
@@ -676,7 +767,7 @@ snd_dev_t **snd_get_devices(int *dev_count)
         p_di = Pa_GetDeviceInfo(i);
         if(p_di == NULL)
         {
-            snprintf(info_buf, sizeof(info_buf), "Error getting device Info (%d)", i);
+            snprintf(info_buf, sizeof(info_buf), _("Error getting device Info (%d)"), i);
             print_info(info_buf, 1);
             continue;
         }
@@ -685,17 +776,19 @@ snd_dev_t **snd_get_devices(int *dev_count)
         //Save only devices which have input Channels
         if(p_di->maxInputChannels <= 0)
             continue;
+        
 
+        const PaHostApiInfo *pa_hostapi = Pa_GetHostApiInfo(p_di->hostApi);
         pa_params.device = i;
-        pa_params.channelCount = cfg.audio.channel;
+        pa_params.channelCount = p_di->maxInputChannels;
         pa_params.sampleFormat = paInt16;
         pa_params.suggestedLatency = p_di->defaultHighInputLatency;
         pa_params.hostApiSpecificStreamInfo = NULL;
 
         //add the supported samplerates to the device structure
-        for(j = 0; j < 9; j++)
+        for(j = 0; j < sizeof(sr)/sizeof(sr[0]); j++)
         {
-            if(Pa_IsFormatSupported(&pa_params, NULL, sr[j]) != paInvalidSampleRate)
+            if(Pa_IsFormatSupported(&pa_params, NULL, sr[j]) == paFormatIsSupported)
             {
                 dev_list[dev_num]->sr_list[sr_count] = sr[j];
                 sr_count++;
@@ -711,9 +804,11 @@ snd_dev_t **snd_get_devices(int *dev_count)
         //Mark the end of the samplerate list for this device with a 0
         dev_list[dev_num]->sr_list[sr_count] = 0;
 
-        dev_list[dev_num]->name = (char*) malloc(strlen(p_di->name)+1);
-        strcpy(dev_list[dev_num]->name, p_di->name);
+        dev_list[dev_num]->name = (char*) malloc(strlen(p_di->name)+strlen(pa_hostapi->name)+10);
         dev_list[dev_num]->dev_id = i;
+        dev_list[dev_num]->num_of_channels = p_di->maxInputChannels;
+        snprintf(dev_list[dev_num]->name, strlen(p_di->name)+strlen(pa_hostapi->name)+10, "%d: %s [%s]", dev_num, p_di->name, pa_hostapi->name);
+
 
         //copy the sr_list from the device where the
         //virtual default device points to
@@ -723,6 +818,7 @@ snd_dev_t **snd_get_devices(int *dev_count)
                     sizeof(dev_list[dev_num]->sr_list));
             
             dev_list[0]->num_of_sr = sr_count;
+            dev_list[0]->num_of_channels = p_di->maxInputChannels;
         }
 
 
@@ -733,7 +829,6 @@ snd_dev_t **snd_get_devices(int *dev_count)
         dev_num++;
     }//for(i = 0; i < devcount && i < 100; i++)
 
-
     if(dev_num == 1)
         *dev_count = 0;
     else
@@ -757,7 +852,7 @@ void snd_reset_samplerate_conv(int rec_or_stream)
         srconv_state_stream = src_new(cfg.audio.resample_mode, cfg.audio.channel, &error);
         if (srconv_state_stream == NULL)
         {
-            print_info("ERROR: Could not initialize samplerate converter", 0);
+            print_info(_("ERROR: Could not initialize samplerate converter"), 0);
         }
     }
 
@@ -773,11 +868,33 @@ void snd_reset_samplerate_conv(int rec_or_stream)
         srconv_state_record = src_new(cfg.audio.resample_mode, cfg.audio.channel, &error);
         if (srconv_state_record == NULL)
         {
-            print_info("ERROR: Could not initialize samplerate converter", 0);
+            print_info(_("ERROR: Could not initialize samplerate converter"), 0);
         }
     }
 }
 
+char *snd_get_device_name(int id)
+{
+    const PaDeviceInfo *pa_dev_info;
+    
+    pa_dev_info = Pa_GetDeviceInfo(id);
+    if(pa_dev_info == NULL)
+        return NULL;
+    else
+        return strdup(pa_dev_info->name); // Make sure it will be free'd on the caller side
+}
+
+int snd_get_number_of_channels(int id)
+{
+    const PaDeviceInfo *pa_dev_info;
+    
+    pa_dev_info = Pa_GetDeviceInfo(id);
+    if(pa_dev_info == NULL)
+        return 0;
+    else
+        return pa_dev_info->maxInputChannels;
+}
+
 void snd_close(void)
 {
     Pa_StopStream(stream);
