@@ -29,7 +29,9 @@ typedef struct
     int dev_id;
     int sr_list[10]; //8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000, 0
     int num_of_sr;
-
+    int num_of_channels;
+    int left_ch; //<- implementieren
+    int right_ch;
 }snd_dev_t;
 
 enum {
@@ -42,6 +44,9 @@ extern bool try_to_connect;
 extern bool pa_new_frames;
 extern bool reconnect;
 extern bool next_file;
+extern bool silence_detected;
+extern bool signal_detected;
+
 extern FILE *next_fd;
 
 int *snd_get_samplerates(int *sr_count);
@@ -49,7 +54,6 @@ snd_dev_t **snd_get_devices(int *dev_count);
 void *snd_rec_thread(void *data);
 void *snd_stream_thread(void *data);
 
-void snd_loop(void);
 void snd_close(void);
 void snd_update_vu(void);
 void snd_start_stream(void);
