@@ -17,7 +17,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef _WIN32
+#ifdef WIN32
  #define usleep(us) Sleep(us/1000)
 #else
  #include <sys/types.h>
@@ -29,6 +29,8 @@
 
 #include <errno.h>
 
+#include "gettext.h"
+
 #include "cfg.h"
 #include "butt.h"
 #include "timer.h"
@@ -51,7 +53,7 @@ void send_icy_header(char *key, char *val)
     icy_line = (char*)malloc(len * sizeof(char)+1);
     snprintf(icy_line, len, "%s:%s\r\n", key, val);
     
-    sock_send(&stream_socket, icy_line, strlen(icy_line), SEND_TIMEOUT);
+    sock_send(stream_socket, icy_line, strlen(icy_line), SEND_TIMEOUT);
 
     free(icy_line);
 }
@@ -73,42 +75,45 @@ int sc_connect(void)
             case SOCK_ERR_CREATE:
                 if(!error_printed)
                 {
-                    print_info("\nConnect: Could not create network socket", 1);
+                    print_info(_("\nConnect: Could not create network socket"), 1);
                     error_printed = 1;
                 }
-                ret = 2;
+                if (cfg.main.force_reconnecting == 1)
+                    ret = SC_RETRY;
+                else
+                    ret = SC_ABORT;
                 break;
             case SOCK_ERR_RESOLVE:
                 if(!error_printed)
                 {
-                    print_info("\nConnect: Error resolving server address", 1);
+                    print_info(_("\nConnect: Error resolving server address"), 1);
                     error_printed = 1;
                 }
-                ret = 1;
+                ret = SC_RETRY;
                 break;
             case SOCK_TIMEOUT:
             case SOCK_INVALID:
-                ret = 1;
+                ret = SC_RETRY;
                 break;
             default:
-                ret = 2;
+                if (cfg.main.force_reconnecting == 1)
+                    ret = SC_RETRY;
+                else
+                    ret = SC_ABORT;
         }
 
-        sc_disconnect();
         return ret;
     }
-
-    error_printed = 0;
     
     snprintf(send_buf, sizeof(send_buf), "%s%s", cfg.srv[cfg.selected_srv]->pwd, "\r\n");
-    sock_send(&stream_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
+    sock_send(stream_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
     
     // Make butt compatible to proxies/load balancers. Thanks to boyska
     if(cfg.srv[cfg.selected_srv]->port == 80)
         snprintf(send_buf, sizeof(send_buf), "Host: %s\r\n", cfg.srv[cfg.selected_srv]->addr);
     else
         snprintf(send_buf, sizeof(send_buf), "Host: %s:%d\r\n", cfg.srv[cfg.selected_srv]->addr, cfg.srv[cfg.selected_srv]->port);
-    sock_send(&stream_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
+    sock_send(stream_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
 
 
    if(cfg.main.num_of_icy > 0)
@@ -133,7 +138,7 @@ int sc_connect(void)
 
 
     
-    sock_send(&stream_socket, "content-type:", 13, SEND_TIMEOUT);
+    sock_send(stream_socket, "content-type:", 13, SEND_TIMEOUT);
 
     if(!strcmp(cfg.audio.codec, "mp3"))
         strcpy(send_buf, "audio/mpeg");
@@ -142,24 +147,24 @@ int sc_connect(void)
     else
         strcpy(send_buf,  "audio/ogg");
 
-    sock_send(&stream_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
+    sock_send(stream_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
    
     
-    sock_send(&stream_socket, "\r\n\r\n", 4, SEND_TIMEOUT);
+    sock_send(stream_socket, "\r\n\r\n", 4, SEND_TIMEOUT);
 
-    if((ret = sock_recv(&stream_socket, recv_buf, sizeof(recv_buf)-1, 5*RECV_TIMEOUT)) == 0)
+    if((ret = sock_recv(stream_socket, recv_buf, sizeof(recv_buf)-1, 5*RECV_TIMEOUT)) == 0)
     {
         usleep(100*1000);
         sc_disconnect();
-        return 1;
+        return SC_RETRY;
     }
     
     if (ret == SOCK_TIMEOUT)
     {
-        print_info("\nconnect: connection timed out. Trying again...\n", 1);
+        print_info(_("\nconnect: connection timed out. Trying again...\n"), 1);
         usleep(100*1000);
         sc_disconnect();
-        return 1;
+        return SC_RETRY;
     }
 
     recv_buf[ret] = '\0';
@@ -169,26 +174,33 @@ int sc_connect(void)
     {
         if(strstr(recv_buf, "invalid password") != NULL || (strstr(recv_buf, "Invalid Password") != NULL))
         {
-            print_info("\nConnect: Invalid password!\n", 1);
+            if(!error_printed)
+            {
+                print_info(_("\nConnect: Invalid password!\n"), 1);
+                error_printed = 1;
+            }
             sc_disconnect();
-            return 2;
+            if (cfg.main.force_reconnecting == 1)
+                return SC_RETRY;
+            else
+                return SC_ABORT;
         }
-        return 1;
+        return SC_RETRY;
     }
 
 
-
     connected = 1;
+    error_printed = 0;
 
     timer_init(&stream_timer, 1);       //starts the "online" timer
 
-    return 0;
+    return SC_OK;
 }
 
 int sc_send(char *buf, int buf_len)
 {
     int ret;
-    ret = sock_send(&stream_socket, buf, buf_len, SEND_TIMEOUT);
+    ret = sock_send(stream_socket, buf, buf_len, SEND_TIMEOUT);
 
     if(ret == SOCK_TIMEOUT)
         ret = -1;
@@ -196,7 +208,7 @@ int sc_send(char *buf, int buf_len)
     return ret;
 }
 
-int sc_update_song(void)
+int sc_update_song(char *song_name)
 {
     int ret;
     int web_socket;
@@ -211,25 +223,25 @@ int sc_update_song(void)
         switch(web_socket)
         {
             case SOCK_ERR_CREATE:
-                print_info("\nUpdate song: Could not create network socket", 1);
-                ret = 1;
+                print_info(_("\nUpdate song: Could not create network socket"), 1);
+                ret = SC_ABORT;
                 break;
             case SOCK_ERR_RESOLVE:
-                print_info("\nUpdate song: Error resolving server address", 1);
-                ret = 1;
+                print_info(_("\nUpdate song: Error resolving server address"), 1);
+                ret = SC_ABORT;
                 break;
             case SOCK_TIMEOUT:
             case SOCK_INVALID:
-                ret = 1;
+                ret = SC_RETRY;
                 break;
             default:
-                ret = 1;
+                ret = SC_ABORT;
         }
 
         return ret;
     }
 
-    song_buf = strdup(cfg.main.song);
+    song_buf = strdup(song_name);
 
     strrpl(&song_buf, (char*)" ", (char*)"%20", MODE_ALL);
     strrpl(&song_buf, (char*)"&", (char*)"%26", MODE_ALL);
@@ -240,7 +252,7 @@ int sc_update_song(void)
                       song_buf
                      );
 
-    sock_send(&web_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
+    sock_send(web_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
     
     
     if(cfg.srv[cfg.selected_srv]->port == 80)
@@ -248,18 +260,18 @@ int sc_update_song(void)
     else
         snprintf(send_buf, sizeof(send_buf), "Host: %s:%d\r\n\r\n", cfg.srv[cfg.selected_srv]->addr, cfg.srv[cfg.selected_srv]->port);
     
-    sock_send(&web_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
+    sock_send(web_socket, send_buf, strlen(send_buf), SEND_TIMEOUT);
 
     
     
-    sock_close(&web_socket);
+    sock_close(web_socket);
     free(song_buf);
 
-    return 0;
+    return SC_OK;
 }
 
 void sc_disconnect(void)
 {
-    sock_close(&stream_socket);
+    sock_close(stream_socket);
 }
 
