@@ -16,8 +16,14 @@
 #ifndef SHOUTCAST_H 
 #define SHOUTCAST_H
 
+enum {
+    SC_OK = 0,
+    SC_RETRY = 1,
+    SC_ABORT = 2,
+    SC_ASK = 3,
+};
 
-int sc_update_song(void);
+int sc_update_song(char *song_name);
 int sc_connect(void);
 int sc_send(char *buf, int buf_len);
 void sc_disconnect(void);
