@@ -29,7 +29,9 @@ enum {
     SOCK_INVALID = -4,
     SOCK_NO_MODE = -5,
     SOCK_ERR_SET_SBUF = -6,
-    SOCK_ERR_SET_RBUF = -7
+    SOCK_ERR_SET_RBUF = -7,
+    SOCK_ERR_BIND = -8,
+    SOCK_ERR_LISTEN = -9
 };
 
 enum {
@@ -40,17 +42,18 @@ enum {
 
 
 int sock_connect(const char *addr, short port, int timout_ms);
-int sock_setbufsize(int *s, int send_size, int recv_size);
-int sock_isdisconnected(int *s);
-int sock_send(int *s, const char *buf, int len, int timout_ms);
-int sock_recv(int *s, char *buf, int len, int timout_ms);
-int sock_select(int *s, int timout_ms, int mode);
-int sock_nonblock(int *s);
-int sock_block(int *s);
-int sock_isvalid(int *s);
-void sock_close(int *s);
-void sock_fdinit(int *s);
-void sock_fdclr(int *s);
+int sock_listen(void);
+int sock_setbufsize(int s, int send_size, int recv_size);
+int sock_isdisconnected(int s);
+int sock_send(int s, const char *buf, int len, int timout_ms);
+int sock_recv(int s, char *buf, int len, int timout_ms);
+int sock_select(int s, int timout_ms, int mode);
+int sock_nonblock(int s);
+int sock_block(int s);
+int sock_isvalid(int s);
+void sock_close(int s);
+void sock_fdinit(int s);
+void sock_fdclr(int s);
 void sock_fdzero(void);
 
 
