@@ -24,6 +24,7 @@ void timer_init(sec_timer *t, int duration)
     t->start_time = time(NULL);
     t->new_time = t->start_time;
     t->duration = duration;
+    t->is_running = true;
 }
 
 int timer_is_elapsed(sec_timer *t)
@@ -56,3 +57,10 @@ char *timer_get_time_str(sec_timer *t)
     return time_str;
 }
 
+void timer_reset(sec_timer *t)
+{
+    t->is_running = false;
+    t->start_time = time(NULL);
+    t->new_time = t->start_time;
+}
+
