@@ -39,6 +39,7 @@ int vorbis_enc_init(vorbis_enc *vorbis)
 
     vorbis_comment_init(&(vorbis->vc));
     vorbis_comment_add_tag(&(vorbis->vc), "ENCODER", PACKAGE_STRING);
+//    vorbis_comment_add_tag(&(vorbis->vc), "TITLE", "Hello Song");
 
     vorbis_analysis_init(&(vorbis->vd), &(vorbis->vi));
     vorbis_block_init(&(vorbis->vd), &(vorbis->vb));
@@ -59,6 +60,7 @@ void vorbis_enc_write_header(vorbis_enc *vorbis)
 
     vorbis_analysis_headerout(&(vorbis->vd), &(vorbis->vc),
             &header, &header_comm, &header_code);
+
     ogg_stream_packetin(&(vorbis->os), &header);
     ogg_stream_packetin(&(vorbis->os), &header_comm);
     ogg_stream_packetin(&(vorbis->os), &header_code);
@@ -84,20 +86,14 @@ int vorbis_enc_encode(vorbis_enc *vorbis, short *pcm_buf, char *enc_buf, int siz
     if(size == 0)
         return 0;
 
-
     /* This ensures the actual
      * audio data will start on a new page, as per spec
      */
-    while(!eos)
-    {
-        result = ogg_stream_flush(&(vorbis->os), &(vorbis->og));
-        if(result == 0)
-            break;
-        memcpy(enc_buf+w, vorbis->og.header, vorbis->og.header_len);
-        w += vorbis->og.header_len;
-        memcpy(enc_buf+w, vorbis->og.body, vorbis->og.body_len);
-        w += vorbis->og.body_len;
-    }
+    result = ogg_stream_flush(&(vorbis->os), &(vorbis->og));
+    memcpy(enc_buf+w, vorbis->og.header, vorbis->og.header_len);
+    w += vorbis->og.header_len;
+    memcpy(enc_buf+w, vorbis->og.body, vorbis->og.body_len);
+    w += vorbis->og.body_len;
 
     vorbis_buf = vorbis_analysis_buffer(&(vorbis->vd), size);
 
@@ -130,18 +126,19 @@ int vorbis_enc_encode(vorbis_enc *vorbis, short *pcm_buf, char *enc_buf, int siz
             /* weld the packet into the bitstream */
             ogg_stream_packetin(&(vorbis->os),&(vorbis->op));
 
+
             /* write out pages (if any) */
             while(!eos)
             {
                 result = ogg_stream_pageout(&(vorbis->os), &(vorbis->og));
                 if(result == 0) 
                     break;
+
                 memcpy(enc_buf+w, vorbis->og.header, vorbis->og.header_len);
                 w += vorbis->og.header_len;
                 memcpy(enc_buf+w, vorbis->og.body, vorbis->og.body_len);
                 w += vorbis->og.body_len;
-                if(ogg_page_eos(&(vorbis->og)))
-                    eos=1;
+                eos = ogg_page_eos(&(vorbis->og));
             }
         }
     }
