@@ -13,6 +13,8 @@
 // GNU General Public License for more details.
 //
 
+#define _FILE_OFFSET_BITS 64
+
 #include <stdio.h>
 #include <string.h>
 
@@ -20,16 +22,22 @@
 
 int wav_write_header(FILE *fd, short ch, int srate, short bps)
 {
-    long int cur_size;
-    wav_hdr hdr;
+    off_t file_size;
+    uint32_t wav_size;
+    wav_hdr_t hdr;
 
-    cur_size = ftell(fd);
+    file_size = ftello(fd);
+    if (file_size == -1)
+        return -1;
+    
+    wav_size = file_size > UINT32_MAX ? UINT32_MAX : (uint32_t)file_size;
+    
+    hdr.wav.riff_size = wav_size >= 44 ? (uint32_t)(wav_size-8) : 0;
 
-    hdr.wav.riff_size = cur_size >= 44 ? cur_size-8 : 0;
     memcpy(&hdr.wav.riff_id, "RIFF", 4);
     memcpy(&hdr.wav.riff_format, "WAVE", 4);
-
     memcpy(hdr.wav.fmt_id, "fmt ", 4);
+    
     hdr.wav.fmt_size = 16;
     hdr.wav.fmt_format = 1;
     hdr.wav.fmt_channel = ch;
@@ -39,14 +47,14 @@ int wav_write_header(FILE *fd, short ch, int srate, short bps)
     hdr.wav.fmt_byte_rate = srate * hdr.wav.fmt_block_align;
 
     memcpy(&hdr.wav.data_id, "data", 4);
-    hdr.wav.data_size = cur_size >= 44 ? cur_size-44 : 0;
+    hdr.wav.data_size = wav_size >= 44 ? (uint32_t)(wav_size-44) : 0;
 
-    //write the header to the beginning of the file
+    //write header to the beginning of the file
     rewind(fd);
-    fwrite(&hdr.data, 1, sizeof(hdr.data), fd);
+    fwrite(&hdr.data, 1, sizeof(wav_hdr_t), fd);
 
     //set the fd back to the fileend
-    fseek(fd, cur_size, SEEK_SET);
+    fseeko(fd, file_size, SEEK_SET);
 
     return 0;
 }
